#include <stdio.h>
#include <math.h>
#include <unistd.h>

#include "coords.h"
#include "tr.h"
#include "msd.h"

typedef struct Flags { 
    unsigned int verbose   : 4 ; 
} Flags ;

static void
usage ()
{
    cbanner ( "$Date: 2007/11/08 17:15:46 $", 
	     "[-T tolerance] [-vV] miniseed|dir [msd2|dir2 ...] database",
	     "** source code example from BRTT **", 
	     "     $ANTELOPE/example/c/tr",
	     "support@brtt.com" ) ; 
    exit (1);
}

typedef struct Params { 
    Dbptr db ; 
    Msd *msd ; 
} Params ;

static int MAXDIR, MAXDFILE ; 

int msd2db (char *path, struct stat *statbuf, void *vp)
{
    Params *p ;
    int size, retcode = 0 ;
    Dbptr db ;
    char *data ; 
    Msd *msd ;
    char dir[FILENAME_MAX], dfile[FILENAME_MAX] ; 
    int bad = 0 ; 

    p = (Params *) vp ; 
    db = p->db ; 
    msd = p->msd ;
    
    size = mapfile ( path, 0,  (char **) &data ) ; 
    if ( size > 0 ) { 
	printf ( "%-15s %10.3f Mbytes\n", path, Mbytes(size) ) ; 
	msdput ( msd, MSD_FILENAME, path, 
		    MSD_MSEED, data, 
		    MSD_MSEED_NBYTES, size, 
		    0 ) ;
	if ( msd2wf ( msd, db, 0 ) ) { 
	    complain ( 0, "some problem in %s\n", path ) ; 
	    retcode = 1 ;
	}
	unmapfile (path, 0) ;
    } else { 
	complain ( 1, "Can't mmap %s", path ) ; 
	retcode = 1 ; 
    }
    return retcode ;
}

int
main (int argc, char **argv)
{
    int             c,
                    errflg = 0;
    Dbptr           db ;
    char           *database;
    Msd		   *msd ;
    Flags	    flags ;
    Params 	    p ;
    int		    n0, n1 ;

    Program_Name = argv[0];

    memset ( &flags, 0, sizeof(flags) ) ; 

    while ((c = getopt (argc, argv, "T:vV")) != -1) {
	switch (c) {
	case 'T':
	    trTOLERANCE = atof(optarg); 
	    elog_notify ( 0, "Using %.3f for tick tolerance; time tags may be wrong\n", 
	    	trTOLERANCE ) ; 
	    break ;

	case 'v':
	    flags.verbose = 1;
	    break;

	case 'V':
	    usage() ;
	    exit (0);

	default:
	    errflg++;
	}
    }

    if (errflg || argc - optind < 2 ) 
	usage ();

    database = argv[argc-1];
    if (dbopen_database (database, "r+", &db) < 0)
	die (0, "Can't open database %s\n", database);

    finit_db(db) ;

    if (db.table == dbALL)
	db = dblookup (db, 0, "wfdisc", 0, 0);

    dbquery(db, dbRECORD_COUNT, &n0 ) ; 
    db = dblookup (db, 0, 0, "dir", 0) ;
    dbquery (db, dbFIELD_SIZE, &MAXDIR) ; 
    db = dblookup (db, 0, 0, "dfile", 0) ;
    dbquery (db, dbFIELD_SIZE, &MAXDFILE) ; 

    msd = msdnew() ;
    p.msd = msd ; 
    p.db = db ;
    for (; optind < argc-1; optind++) {
	char *path ;
	path = argv[optind] ;
	if ( is_file (path) ) {
	    msd2db(path, 0, (void *) &p) ;
	} else if ( is_dir (path) ) { 
	    fstraverse(path, FS_ONLY_FILE, msd2db, (void *) &p) ;
	} else { 
	    complain (0, "Can't deal with %s", path ) ; 
	}
    }
    msdfree(msd) ; 
    dbquery(db, dbRECORD_COUNT, &n1 ) ; 

    if ( n1-n0 > 0 ) { 
	printf ( "%d new wfdisc records\n", n1 - n0 ) ; 
    } else { 
	complain ( 0, "No miniseed data found\n" ) ; 
    }

    dbclose(db) ; 

    return 0;
}

